/*
More informations about this Project can be found at : https://hackaday.io/project/191531-2-cans-one-arduino

This example generates a can message with a random ID, random length and random data and sends it out on CAN1.
Details about the generated message will be printed out over serial.
As soon as CAN0 receives a message, details about it will be printed out over serial.

If your construction works you should see 2 nearly identical outputs over serial when you connect CAN0 and CAN1 together.
something like
Sent a message on CAN1 ID : 0x238 with a length of 5 bytes. The data was : 0xB2 0xBB 0xCD 0xEE 0x32 
 Got a message on CAN0 ID : 0x238 with a length of 5 bytes. The data was : 0xB2 0xBB 0xCD 0xEE 0x32 

 This example uses the great "MCP_CAN_lib" by Cory J. Fowler.

*/

#include <mcp_can.h>  // include can library
MCP_CAN CAN0(10);     // CAN0 interface usins CS on digital pin 10
MCP_CAN CAN1(9);      // CAN1 interface using CS on digital pin 9
#define CAN0_INT 2    // define interrupt pin for CAN0 receive buffer
#define CAN1_INT 3    // define interrupt pin for CAN1 receive buffer

// some variables for the timer
unsigned long prevMillis = 0;  // timer stuff

// CAN TX Variables
byte txBuf[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };  // prepare a empty CAN message to send
long unsigned int txId;
unsigned char txlen;

// CAN RX Variables
long unsigned int rxId;
unsigned char rxlen;
unsigned char rxBuf[8];



void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);                        // start serial
  pinMode(CAN0_INT, INPUT_PULLUP);             // set up INT Pin for can0
  CAN0.begin(MCP_ANY, CAN_500KBPS, MCP_8MHZ);  // set up can0 at 500k baudrate and 8MHz (works fine on UNO R3 with 16MHz clock)
  CAN0.setMode(MCP_NORMAL);                    // set up can0 in normal mode
  pinMode(CAN1_INT, INPUT_PULLUP);             // set up INT Pin for can1
  CAN1.begin(MCP_ANY, CAN_500KBPS, MCP_8MHZ);  // set up can1 at 500k baudrate and 8MHz (works fine on UNO R3 with 16MHz clock)
  CAN1.setMode(MCP_NORMAL);                    // set up can1 in normal mode
}

void loop() {
  // put your main code here, to run repeatedly:

  ////////////////////////////////////////////////////////////////////////////////////////////////////////
  // this code will only execute once every 500 milliseconds
  unsigned long currMillis = millis();   // put the current time into the currMillis variable
  if (currMillis - prevMillis >= 500) {  // this code will execute if 500 milliseconds have passed
    prevMillis = currMillis;             // reset the timer

    /*
CANx.sendMsgBuf(ID, EXT, DLC, DATA)

CANx is the CAN (0 or 1) we have defined in the setup

ID is the CAN Address
The highest ID in normal(0) mode is 0x7FF
The highest ID in extended(1) mode is 0x1FFFFFFF

EXT is a switch to use the normal(0) or extended(1) ID range

DLC defines how many Bytes our sent message will have (1 to 8)

DATA is the actual message we want to send.

*/
    // generate 8 Bytes of random CAN data to send
    for (int i = 0; i < 8; i++) {  // ecexute this "for loop" 8 times. Variable i is used as a counter and also as indicator wich byte of txBuf to use
      txBuf[i] = random(0, 256);   // generate a random number from 0 to 255 (its 256 because the max random number will be 1 below that... 0xFF)
    }

    txId = random(0x000, 0x800);             // generate a random 11bit CAN ID (0x000 to 0x7FF)
    txlen = random(1, 9);                    // generate a random count of bytes to send (1 - 8)


// generate some serial output with details to the message we just sent on CAN1
    Serial.print("Sent a message on CAN1 ID : ");
    Serial.print("0x");
    if (txId <= (0x00F)) {
      Serial.print("0");
    }
    if (txId <= (0x0FF)) {
      Serial.print("0");
    }
    Serial.print(txId, HEX);
    Serial.print(" with a length of ");
    Serial.print(txlen);
    Serial.print(" bytes. ");
    Serial.print("The data was : ");

    for (int i = 0; i < txlen; i++) {  
      Serial.print("0x");
      if (txBuf[i] <= (0x0F)) {
        Serial.print("0");
      }
      Serial.print(txBuf[i], HEX);
      Serial.print(" ");
    }
    Serial.println("");
        CAN1.sendMsgBuf(txId, 0, txlen, txBuf);  // send the random message we just generated above

  }


  ////////////////////////////////////////////////////////////////////////////////////////////////////////
  // listen on CAN0 if there is any data. If yes print some informations about the message out on serial

  if (!digitalRead(CAN0_INT)) {             // If interrupt pin is low, read CAN0 receive buffer
    CAN0.readMsgBuf(&rxId, &rxlen, rxBuf);  // Read buffer and put it into : ID, length , Data storage

// generate some serial output with details to the message we just sent on CAN1
    Serial.print(" Got a message on CAN0 ID : ");
    Serial.print("0x");
    if (rxId <= (0x00F)) {
      Serial.print("0");
    }
    if (rxId <= (0x0FF)) {
      Serial.print("0");
    }
    Serial.print(rxId, HEX);
    Serial.print(" with a length of ");
    Serial.print(rxlen);
    Serial.print(" bytes. ");
    Serial.print("The data was : ");

    for (int i = 0; i < rxlen; i++) {  
      Serial.print("0x");
      if (rxBuf[i] <= (0x0F)) {
        Serial.print("0");
      }
      Serial.print(rxBuf[i], HEX);
      Serial.print(" ");
    }
    Serial.println("");


  }
}
